@@ -97,7 +97,7 @@ typedef struct
     faidx_t *fai;
     struct { int tot, set, swap; } nref;
     char **argv, *output_fname, *ref_fname, *vcf_fname, *region, *targets;
-    int argc, rmdup, output_type, n_threads, check_ref, strict_filter, do_indels;
+    int argc, rmdup, output_type, n_threads, check_ref, strict_filter, do_indels, clevel;
     int nchanged, nskipped, nsplit, ntotal, mrows_op, mrows_collapse, parsimonious;
     int record_cmd_line, force, force_warned, keep_sum_ad;
     abuf_t *abuf;
@@ -336,7 +336,7 @@ static void set_old_rec_tag(args_t *args, bcf1_t *dst, bcf1_t *src, int ialt)
     }
     if ( ialt>0 )
     {
-        args->tmp_kstr.s[args->tmp_kstr.l-1] = '|';
+        kputc('|',&args->tmp_kstr);
         kputw(ialt,&args->tmp_kstr);
     }
     if ( (bcf_update_info_string(args->out_hdr, dst, args->old_rec_tag, args->tmp_kstr.s))!=0 )
@@ -1252,7 +1252,7 @@ static void merge_format_genotype(args_t *args, bcf1_t **lines, int nlines, bcf_
     int nsmpl = bcf_hdr_nsamples(args->hdr);
     ngts /= nsmpl;
 
-    int i, j, k;
+    int i, j, k,k2;
     for (i=1; i<nlines; i++)
     {
         int ntmp2 = args->ntmp_arr2 / 4;
@@ -1265,16 +1265,19 @@ static void merge_format_genotype(args_t *args, bcf1_t **lines, int nlines, bcf_
         int32_t *gt2 = (int32_t*) args->tmp_arr2;
         for (j=0; j<nsmpl; j++)
         {
-            for (k=0; k<ngts; k++)
+            for (k2=0; k2<ngts2; k2++)
             {
-                if ( gt2[k]==bcf_int32_vector_end ) break;
-                if ( bcf_gt_is_missing(gt2[k]) || bcf_gt_allele(gt2[k])==0 ) continue;
-                if ( gt2[k]==0 ) gt[k] = 0; // missing genotype
-                else
+                if ( gt2[k2]==bcf_int32_vector_end ) break;
+                if ( bcf_gt_is_missing(gt2[k2]) ) continue;
+                int ial2 = bcf_gt_allele(gt2[k2]);
+                if ( ial2==0 ) continue;    // never overwrite with ref
+                if ( ial2>=args->maps[i].nals ) error("Error at %s:%"PRId64": incorrect allele index %d\n",bcf_seqname(args->hdr,lines[i]),(int64_t) lines[i]->pos+1,ial2);
+                int ial = args->maps[i].map[ial2];
+                for (k=0; k<ngts; k++)
+                    if ( gt[k]==bcf_int32_vector_end || bcf_gt_is_missing(gt[k]) || !bcf_gt_allele(gt[k]) ) break;
+                if ( k<ngts )
                 {
-                    int ial = bcf_gt_allele(gt2[k]);
-                    if ( ial>=args->maps[i].nals ) error("Error at %s:%"PRId64": incorrect allele index %d\n",bcf_seqname(args->hdr,lines[i]),(int64_t) lines[i]->pos+1,ial);
-                    gt[k] = bcf_gt_unphased( args->maps[i].map[ial] ) | bcf_gt_is_phased(gt[k]);
+                    gt[k] = bcf_gt_unphased(ial);
                 }
             }
             gt  += ngts;
@@ -1987,7 +1990,9 @@ static bcf1_t *next_atomized_line(args_t *args)
 }
 static void normalize_vcf(args_t *args)
 {
-    args->out = hts_open(args->output_fname, hts_bcf_wmode2(args->output_type,args->output_fname));
+    char wmode[8];
+    set_wmode(wmode,args->output_type,args->output_fname,args->clevel);
+    args->out = hts_open(args->output_fname ? args->output_fname : "-", wmode);
     if ( args->out == NULL ) error("Can't write to \"%s\": %s\n", args->output_fname, strerror(errno));
     if ( args->n_threads )
         hts_set_opt(args->out, HTS_OPT_THREAD_POOL, args->files->p);
@@ -2085,12 +2090,14 @@ static void usage(void)
     fprintf(stderr, "    -N, --do-not-normalize          Do not normalize indels (with -m or -c s)\n");
     fprintf(stderr, "        --old-rec-tag STR           Annotate modified records with INFO/STR indicating the original variant\n");
     fprintf(stderr, "    -o, --output FILE               Write output to a file [standard output]\n");
-    fprintf(stderr, "    -O, --output-type TYPE          'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
+    fprintf(stderr, "    -O, --output-type u|b|v|z[0-9]  u/b: un/compressed BCF, v/z: un/compressed VCF, 0-9: compression level [v]\n");
     fprintf(stderr, "    -r, --regions REGION            Restrict to comma-separated list of regions\n");
     fprintf(stderr, "    -R, --regions-file FILE         Restrict to regions listed in a file\n");
+    fprintf(stderr, "        --regions-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [1]\n");
     fprintf(stderr, "    -s, --strict-filter             When merging (-m+), merged site is PASS only if all sites being merged PASS\n");
     fprintf(stderr, "    -t, --targets REGION            Similar to -r but streams rather than index-jumps\n");
     fprintf(stderr, "    -T, --targets-file FILE         Similar to -R but streams rather than index-jumps\n");
+    fprintf(stderr, "        --targets-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n");
     fprintf(stderr, "        --threads INT               Use multithreading with <int> worker threads [0]\n");
     fprintf(stderr, "    -w, --site-win INT              Buffer for sorting lines which changed position during realignment [1000]\n");
     fprintf(stderr, "\n");
@@ -2118,9 +2125,12 @@ int main_vcfnorm(int argc, char *argv[])
     args->buf_win = 1000;
     args->mrows_collapse = COLLAPSE_BOTH;
     args->do_indels = 1;
+    args->clevel = -1;
     int region_is_file  = 0;
     int targets_is_file = 0;
     args->use_star_allele = 1;
+    int regions_overlap = 1;
+    int targets_overlap = 0;
 
     static struct option loptions[] =
     {
@@ -2135,8 +2145,10 @@ int main_vcfnorm(int argc, char *argv[])
         {"multiallelics",required_argument,NULL,'m'},
         {"regions",required_argument,NULL,'r'},
         {"regions-file",required_argument,NULL,'R'},
+        {"regions-overlap",required_argument,NULL,1},
         {"targets",required_argument,NULL,'t'},
         {"targets-file",required_argument,NULL,'T'},
+        {"targets-overlap",required_argument,NULL,2},
         {"site-win",required_argument,NULL,'w'},
         {"remove-duplicates",no_argument,NULL,'D'},
         {"rm-dup",required_argument,NULL,'d'},
@@ -2200,7 +2212,16 @@ int main_vcfnorm(int argc, char *argv[])
                     case 'u': args->output_type = FT_BCF; break;
                     case 'z': args->output_type = FT_VCF_GZ; break;
                     case 'v': args->output_type = FT_VCF; break;
-                    default: error("The output type \"%s\" not recognised\n", optarg);
+                    default:
+                    {
+                        args->clevel = strtol(optarg,&tmp,10);
+                        if ( *tmp || args->clevel<0 || args->clevel>9 ) error("The output type \"%s\" not recognised\n", optarg);
+                    }
+                }
+                if ( optarg[1] )
+                {
+                    args->clevel = strtol(optarg+1,&tmp,10);
+                    if ( *tmp || args->clevel<0 || args->clevel>9 ) error("Could not parse argument: --compression-level %s\n", optarg+1);
                 }
                 break;
             case 'o': args->output_fname = optarg; break;
@@ -2221,6 +2242,14 @@ int main_vcfnorm(int argc, char *argv[])
             case  9 : args->n_threads = strtol(optarg, 0, 0); break;
             case  8 : args->record_cmd_line = 0; break;
             case  7 : args->force = 1; break;
+            case  1 :
+                regions_overlap = parse_overlap_option(optarg);
+                if ( regions_overlap < 0 ) error("Could not parse: --regions-overlap %s\n",optarg);
+                break;
+            case  2 :
+                targets_overlap = parse_overlap_option(optarg);
+                if ( targets_overlap < 0 ) error("Could not parse: --targets-overlap %s\n",optarg);
+                break;
             case 'h':
             case '?': usage(); break;
             default: error("Unknown argument: %s\n", optarg);
@@ -2241,11 +2270,13 @@ int main_vcfnorm(int argc, char *argv[])
 
     if ( args->region )
     {
+        bcf_sr_set_opt(args->files,BCF_SR_REGIONS_OVERLAP,regions_overlap);
         if ( bcf_sr_set_regions(args->files, args->region,region_is_file)<0 )
             error("Failed to read the regions: %s\n", args->region);
     }
     if ( args->targets )
     {
+        bcf_sr_set_opt(args->files,BCF_SR_TARGETS_OVERLAP,targets_overlap);
         if ( bcf_sr_set_targets(args->files, args->targets,targets_is_file, 0)<0 )
             error("Failed to read the targets: %s\n", args->targets);
     }
